package orderservicelogic

import (
	"app-cloudep-trade-service/internal/domain"
	"app-cloudep-trade-service/internal/domain/usecase"
	"context"

	ers "code.30cm.net/digimon/library-go/errs"

	"app-cloudep-trade-service/gen_result/pb/trade"
	"app-cloudep-trade-service/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type ListOrderLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewListOrderLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListOrderLogic {
	return &ListOrderLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

type GetOrderListReq struct {
	PageIndex int64 `json:"page_index" validate:"required"`
	PageSize  int64 `json:"page_size" validate:"required"`

	ReferenceID   string  `json:"reference_id"`
	ReferenceUID  string  `json:"reference_uid"`
	BusinessID    string  `json:"business_id"`
	UID           string  `json:"uid"`
	OrderType     int     `json:"order_type"`
	DirectionType []int64 `json:"direction_type"`
	OrderStatus   []int64 `json:"order_status"`

	StartCreateTime       int64 `json:"start_create_time"`
	EndCreateTime         int64 `json:"end_create_time"`
	StartUpdateTime       int64 `json:"start_update_time"`
	EndUpdateTime         int64 `json:"end_update_time"`
	StartOrderArrivalTime int64 `json:"start_order_arrival_time"`
	EndOrderArrivalTime   int64 `json:"end_order_arrival_time"`
	StartOrderPaymentTime int64 `json:"start_order_payment_time"`
	EndOrderPaymentTime   int64 `json:"end_order_payment_time"`

	CryptoType string `json:"crypto_type"`
	TxHash     string `json:"tx_hash"`
}

// toGetOrderListReq 將 JSON 標籤結構轉換為無 JSON 標籤結構
func toGetOrderListReq(req *trade.ListOrderReq) usecase.GetOrderListReq {
	return usecase.GetOrderListReq{
		PageIndex:             req.PageIndex,
		PageSize:              req.PageSize,
		ReferenceID:           req.ReferenceId,
		ReferenceUID:          req.ReferenceUid,
		BusinessID:            req.BusinessId,
		UID:                   req.Uid,
		OrderType:             domain.OrderType(req.OrderType),
		DirectionType:         i32To64(req.DirectionType),
		OrderStatus:           i32To64(req.OrderStatus),
		StartCreateTime:       req.StartCreateTime,
		EndCreateTime:         req.EndCreateTime,
		StartUpdateTime:       req.StartUpdateTime,
		EndUpdateTime:         req.EndUpdateTime,
		StartOrderArrivalTime: req.StartOrderArrivalTime,
		EndOrderArrivalTime:   req.EndOrderArrivalTime,
		StartOrderPaymentTime: req.StartOrderPaymentTime,
		EndOrderPaymentTime:   req.EndOrderPaymentTime,
		CryptoType:            req.CryptoType,
		TxHash:                req.TxHash,
	}
}

func i32To64(i []int32) []int64 {
	if len(i) == 0 {
		return nil
	}
	result := make([]int64, 0, len(i))
	for item := range i {
		result = append(result, int64(item))
	}

	return result
}

// ListOrder 取得訂單列表
func (l *ListOrderLogic) ListOrder(in *trade.ListOrderReq) (*trade.ListOrderResp, error) {
	// 驗證資料
	if err := l.svcCtx.Validate.ValidateAll(&GetOrderListReq{
		PageIndex: in.GetPageIndex(),
		PageSize:  in.GetPageSize(),
	}); err != nil {
		// 錯誤代碼 06-011-00
		return nil, ers.InvalidFormat(err.Error())
	}

	order, err := l.svcCtx.OrderUseCase.ListOrder(l.ctx, toGetOrderListReq(in))
	if err != nil {
		return nil, err
	}

	data := make([]*trade.GetOrderResp, 0, len(order.Data))
	for _, item := range order.Data {
		data = append(data, &trade.GetOrderResp{
			BusinessId:           item.BusinessID,
			OrderType:            int32(item.OrderType),
			OrderStatus:          int32(item.OrderStatus),
			Brand:                item.Brand,
			OrderUid:             item.OrderUID,
			ReferenceId:          item.ReferenceID,
			Count:                item.Count,
			OrderFee:             item.OrderFee,
			Amount:               item.Amount,
			ReferenceBrand:       item.ReferenceBrand,
			ReferenceUid:         item.ReferenceUID,
			WalletStatus:         item.WalletStatus,
			ThreePartyStatus:     item.ThreePartyStatus,
			DirectionType:        item.DirectionType,
			CryptoType:           item.CryptoType,
			ThirdPartyFee:        item.ThirdPartyFee,
			CryptoToUsdtRate:     item.CryptoToUsdtRate,
			FiatToUsdRate:        item.FiatToUsdRate,
			FeeCryptoToUsdtRate:  item.FeeCryptoToUsdtRate,
			UsdtToCryptoTypeRate: item.UsdtToCryptoTypeRate,
			PaymentFiat:          item.PaymentFiat,
			PaymentUnitPrice:     item.PaymentUnitPrice,
			PaymentTemplateId:    item.PaymentTemplateID,
			OrderArrivalTime:     item.OrderArrivalTime,
			OrderPaymentTime:     item.OrderPaymentTime,
			UnpaidTimeoutSecond:  item.UnpaidTimeoutSecond,
			ChainType:            item.ChainType,
			TxHash:               item.TxHash,
			FromAddress:          item.FromAddress,
			ToAddress:            item.ToAddress,
			ChainFee:             item.ChainFee,
			ChainFeeCrypto:       item.ChainFeeCrypto,
			Memo:                 item.Memo,
			OrderNote:            item.OrderNote,
			CreateTime:           item.CreateTime,
			UpdateTime:           item.UpdateTime,
		})
	}

	return &trade.ListOrderResp{
		Data: data,
		Page: &trade.Pager{
			Index: order.Page.Index,
			Size:  order.Page.Size,
			Total: order.Page.Total,
		},
	}, nil
}
